@@ -9,7 +9,7 @@
 
 class FPDFRenderPatternEmbedderTest : public EmbedderTest {};
 
-TEST_F(FPDFRenderPatternEmbedderTest, LoadError_547706) {
+TEST_F(FPDFRenderPatternEmbedderTest, LoadError547706) {
   // Test shading where object is a dictionary instead of a stream.
   ASSERT_TRUE(OpenDocument("bug_547706.pdf"));
   ScopedEmbedderTestPage page = LoadScopedPage(0);
