@@ -4,8 +4,6 @@
 
 #include <math.h>
 
-#include <vector>
-
 #include "core/fxcrt/fx_string.h"
 #include "fpdfsdk/cpdfsdk_helpers.h"
 #include "fxjs/cjs_event_context.h"
@@ -26,9 +24,9 @@ double RoundDownDate(double date) {
 
 }  // namespace
 
-class CJS_PublicMethodsEmbedderTest : public ExternalEngineEmbedderTest {};
+class CJSPublicMethodsEmbedderTest : public ExternalEngineEmbedderTest {};
 
-TEST_F(CJS_PublicMethodsEmbedderTest, ParseDateUsingFormat) {
+TEST_F(CJSPublicMethodsEmbedderTest, ParseDateUsingFormat) {
   v8::Isolate::Scope isolate_scope(isolate());
   v8::HandleScope handle_scope(isolate());
   v8::Context::Scope context_scope(GetV8Context());
@@ -116,7 +114,7 @@ TEST_F(CJS_PublicMethodsEmbedderTest, ParseDateUsingFormat) {
   EXPECT_FALSE(bWrongFormat);
 }
 
-TEST_F(CJS_PublicMethodsEmbedderTest, PrintDateUsingFormat) {
+TEST_F(CJSPublicMethodsEmbedderTest, PrintDateUsingFormat) {
   v8::Isolate::Scope isolate_scope(isolate());
   v8::HandleScope handle_scope(isolate());
   v8::Context::Scope context_scope(GetV8Context());
@@ -183,13 +181,13 @@ TEST_F(CJS_PublicMethodsEmbedderTest, PrintDateUsingFormat) {
   EXPECT_EQ(L"02012095", formatted_date);
 }
 
-TEST_F(CJS_PublicMethodsEmbedderTest, AFSimple_CalculateSum) {
+TEST_F(CJSPublicMethodsEmbedderTest, AFSimpleCalculateSum) {
   v8::Isolate::Scope isolate_scope(isolate());
   v8::HandleScope handle_scope(isolate());
   v8::Context::Scope context_scope(GetV8Context());
 
   ASSERT_TRUE(OpenDocument("calculate.pdf"));
-  auto* page = LoadPage(0);
+  ScopedEmbedderTestPage page = LoadScopedPage(0);
   ASSERT_TRUE(page);
 
   CJS_Runtime runtime(
@@ -208,7 +206,6 @@ TEST_F(CJS_PublicMethodsEmbedderTest, AFSimple_CalculateSum) {
   params.push_back(ary);
 
   CJS_Result ret = CJS_PublicMethods::AFSimple_Calculate(&runtime, params);
-  UnloadPage(page);
 
   runtime.GetCurrentEventContext()->SetValueForTest(nullptr);
 
@@ -217,13 +214,13 @@ TEST_F(CJS_PublicMethodsEmbedderTest, AFSimple_CalculateSum) {
   ASSERT_EQ(L"7", result);
 }
 
-TEST_F(CJS_PublicMethodsEmbedderTest, AFNumber_Keystroke) {
+TEST_F(CJSPublicMethodsEmbedderTest, AFNumberKeystroke) {
   v8::Isolate::Scope isolate_scope(isolate());
   v8::HandleScope handle_scope(isolate());
   v8::Context::Scope context_scope(GetV8Context());
 
   ASSERT_TRUE(OpenDocument("calculate.pdf"));
-  auto* page = LoadPage(0);
+  ScopedEmbedderTestPage page = LoadScopedPage(0);
   ASSERT_TRUE(page);
 
   CJS_Runtime runtime(
@@ -253,7 +250,6 @@ TEST_F(CJS_PublicMethodsEmbedderTest, AFNumber_Keystroke) {
   EXPECT_TRUE(!ret.HasError());
   EXPECT_TRUE(!ret.HasReturn());
 
-  UnloadPage(page);
 
   // Keep the *SAN bots happy. One of these is an UnownedPtr, another seems to
   // used during destruction. Clear them all to be safe and consistent.
