@@ -4,7 +4,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_GCRYPT_CRYPTO_H__
 #define __XMLSEC_GCRYPT_CRYPTO_H__
@@ -208,23 +208,6 @@ XMLSEC_CRYPTO_EXPORT gcry_sexp_t        xmlSecGCryptKeyDataEcGetPublicKey
 XMLSEC_CRYPTO_EXPORT gcry_sexp_t        xmlSecGCryptKeyDataEcGetPrivateKey      (xmlSecKeyDataPtr data);
 
 
-/**
- * xmlSecGCryptKeyDataEcdsaId:
- *
- * Deprecated. The EC key klass.
- */
-#define xmlSecGCryptKeyDataEcdsaId     xmlSecGCryptKeyDataEcId
-XMLSEC_CRYPTO_EXPORT XMLSEC_DEPRECATED xmlSecKeyDataId    xmlSecGCryptKeyDataEcdsaGetKlass        (void);
-XMLSEC_CRYPTO_EXPORT XMLSEC_DEPRECATED int                xmlSecGCryptKeyDataEcdsaAdoptKey        (xmlSecKeyDataPtr data,
-                                                                                                   gcry_sexp_t ec_key);
-XMLSEC_CRYPTO_EXPORT XMLSEC_DEPRECATED int                xmlSecGCryptKeyDataEcdsaAdoptKeyPair    (xmlSecKeyDataPtr data,
-                                                                                                    gcry_sexp_t pub_key,
-                                                                                                    gcry_sexp_t priv_key);
-XMLSEC_CRYPTO_EXPORT XMLSEC_DEPRECATED gcry_sexp_t        xmlSecGCryptKeyDataEcdsaGetPublicKey    (xmlSecKeyDataPtr data);
-XMLSEC_CRYPTO_EXPORT XMLSEC_DEPRECATED gcry_sexp_t        xmlSecGCryptKeyDataEcdsaGetPrivateKey   (xmlSecKeyDataPtr data);
-
-
-
 #ifndef XMLSEC_NO_SHA1
 /**
  * xmlSecGCryptTransformEcdsaSha1Id:
@@ -553,6 +536,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecGCryptTransformRsaPssSha3_384GetKla
 XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecGCryptTransformRsaPssSha3_512GetKlass(void);
 #endif /* XMLSEC_NO_SHA3 */
 
+#ifndef XMLSEC_NO_RSA_PKCS15
 /**
  * xmlSecGCryptTransformRsaPkcs1Id:
  *
@@ -561,7 +545,9 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecGCryptTransformRsaPssSha3_512GetKla
 #define xmlSecGCryptTransformRsaPkcs1Id \
         xmlSecGCryptTransformRsaPkcs1GetKlass()
 XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecGCryptTransformRsaPkcs1GetKlass(void);
+#endif /* XMLSEC_NO_RSA_PKCS15 */
 
+#ifndef XMLSEC_NO_RSA_OAEP
 /**
  * xmlSecGCryptTransformRsaOaepId:
  *
@@ -579,7 +565,7 @@ XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecGCryptTransformRsaOaepGetKlass(void
 #define xmlSecGCryptTransformRsaOaepEnc11Id \
         xmlSecGCryptTransformRsaOaepEnc11GetKlass()
 XMLSEC_CRYPTO_EXPORT xmlSecTransformId xmlSecGCryptTransformRsaOaepEnc11GetKlass(void);
-
+#endif /* XMLSEC_NO_RSA_OAEP */
 
 #endif /* XMLSEC_NO_RSA */
 
