@@ -58,12 +58,12 @@ EmbFile::EmbFile(Object &&efStream)
         if (paramDict.isDict()) {
             Object paramObj = paramDict.dictLookup("ModDate");
             if (paramObj.isString()) {
-                m_modDate = paramObj.getString()->copy();
+                m_modDate = paramObj.takeString();
             }
 
             paramObj = paramDict.dictLookup("CreationDate");
             if (paramObj.isString()) {
-                m_createDate = paramObj.getString()->copy();
+                m_createDate = paramObj.takeString();
             }
 
             paramObj = paramDict.dictLookup("Size");
@@ -73,7 +73,7 @@ EmbFile::EmbFile(Object &&efStream)
 
             paramObj = paramDict.dictLookup("CheckSum");
             if (paramObj.isString()) {
-                m_checksum = paramObj.getString()->copy();
+                m_checksum = paramObj.takeString();
             }
         }
     }
@@ -124,7 +124,7 @@ FileSpec::FileSpec(const Object *fileSpecA)
         return;
     }
 
-    fileName = obj1.getString()->copy();
+    fileName = obj1.takeString();
 
     if (fileSpec.isDict()) {
         obj1 = fileSpec.dictLookup("EF");
@@ -140,7 +140,7 @@ FileSpec::FileSpec(const Object *fileSpecA)
 
         obj1 = fileSpec.dictLookup("Desc");
         if (obj1.isString()) {
-            desc = obj1.getString()->copy();
+            desc = obj1.takeString();
         }
     }
 }
@@ -196,7 +196,7 @@ GooString *FileSpec::getFileNameForPlatform()
 
     Object obj1 = getFileSpecNameForPlatform(&fileSpec);
     if (obj1.isString()) {
-        platformFileName = obj1.getString()->copy();
+        platformFileName = obj1.takeString();
     }
 
     return platformFileName.get();
@@ -264,7 +264,7 @@ Object getFileSpecNameForPlatform(const Object *fileSpec)
 
     // system-dependent path manipulation
 #ifdef _WIN32
-    int i, j;
+    size_t i, j;
     std::unique_ptr<GooString> name = fileName.getString()->copy();
     // "//...."             --> "\...."
     // "/x/...."            --> "x:\...."
@@ -272,31 +272,31 @@ Object getFileSpecNameForPlatform(const Object *fileSpec)
     // convert escaped slashes to slashes and unescaped slashes to backslashes
     i = 0;
     if (name->getChar(0) == '/') {
-        if (name->getLength() >= 2 && name->getChar(1) == '/') {
-            name->del(0);
+        if (name->size() >= 2 && name->getChar(1) == '/') {
+            name->erase(0, 1);
             i = 0;
-        } else if (name->getLength() >= 2 && ((name->getChar(1) >= 'a' && name->getChar(1) <= 'z') || (name->getChar(1) >= 'A' && name->getChar(1) <= 'Z')) && (name->getLength() == 2 || name->getChar(2) == '/')) {
+        } else if (name->size() >= 2 && ((name->getChar(1) >= 'a' && name->getChar(1) <= 'z') || (name->getChar(1) >= 'A' && name->getChar(1) <= 'Z')) && (name->size() == 2 || name->getChar(2) == '/')) {
             name->setChar(0, name->getChar(1));
             name->setChar(1, ':');
             i = 2;
         } else {
-            for (j = 2; j < name->getLength(); ++j) {
+            for (j = 2; j < name->size(); ++j) {
                 if (name->getChar(j - 1) != '\\' && name->getChar(j) == '/') {
                     break;
                 }
             }
-            if (j < name->getLength()) {
+            if (j < name->size()) {
                 name->setChar(0, '\\');
                 name->insert(0, 1, '\\');
                 i = 2;
             }
         }
     }
-    for (; i < name->getLength(); ++i) {
+    for (; i < name->size(); ++i) {
         if (name->getChar(i) == '/') {
             name->setChar(i, '\\');
-        } else if (name->getChar(i) == '\\' && i + 1 < name->getLength() && name->getChar(i + 1) == '/') {
-            name->del(i);
+        } else if (name->getChar(i) == '\\' && i + 1 < name->size() && name->getChar(i + 1) == '/') {
+            name->erase(i, 1);
         }
     }
     fileName = Object(std::move(name));
