@@ -195,6 +195,10 @@
 #define FB_CPU CpuPowerPc64
 #endif /* PPC64 */
 
+#ifdef M68K
+#define FB_CPU CpuM68k
+#endif /* M68K */
+
 #endif /* LINUX */
 
 
@@ -993,4 +997,7 @@ namespace Firebird {
 	static IMessageMetadata* const DELAYED_OUT_FORMAT = reinterpret_cast<IMessageMetadata*>(1);
 }
 
+#undef UCHAR_TYPE
+#define UCHAR_TYPE uint16_t
+
 #endif /* COMMON_COMMON_H */
