@@ -6,7 +6,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 
 #ifndef __XMLSEC_ERROR_HELPERS_H__
@@ -17,6 +17,7 @@
 #endif /* XMLSEC_PRIVATE */
 
 #include <errno.h>
+#include <xmlsec/exports.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -136,7 +137,7 @@ extern "C" {
  */
 #define xmlSecXmlError(errorFunction, errorObject) \
     {                                                 \
-        xmlErrorPtr error = xmlGetLastError();        \
+        const xmlError * error = xmlGetLastError();        \
         int code = (error != NULL) ? error->code : 0; \
         const char* message = (error != NULL) ? error->message : NULL; \
         xmlSecError(XMLSEC_ERRORS_HERE,               \
@@ -159,7 +160,7 @@ extern "C" {
  */
 #define xmlSecXmlError2(errorFunction, errorObject, msg, param) \
     {                                                 \
-        xmlErrorPtr error = xmlGetLastError();        \
+        const xmlError * error = xmlGetLastError();        \
         int code = (error != NULL) ? error->code : 0; \
         const char* message = (error != NULL) ? error->message : NULL; \
         xmlSecError(XMLSEC_ERRORS_HERE,               \
@@ -181,7 +182,7 @@ extern "C" {
  */
 #define xmlSecXmlParserError(errorFunction, ctxt, errorObject) \
     {                                                 \
-        xmlErrorPtr error = xmlCtxtGetLastError(ctxt);\
+        const xmlError * error = xmlCtxtGetLastError(ctxt);\
         int code = (error != NULL) ? error->code : 0; \
         const char* message = (error != NULL) ? error->message : NULL; \
         xmlSecError(XMLSEC_ERRORS_HERE,               \
@@ -205,7 +206,7 @@ extern "C" {
  */
 #define xmlSecXmlParserError2(errorFunction, ctxt, errorObject, msg, param) \
     {                                                 \
-        xmlErrorPtr error = xmlCtxtGetLastError(ctxt);\
+        const xmlError * error = xmlCtxtGetLastError(ctxt);\
         int code = (error != NULL) ? error->code : 0; \
         const char* message = (error != NULL) ? error->message : NULL; \
         xmlSecError(XMLSEC_ERRORS_HERE,               \
@@ -227,7 +228,7 @@ extern "C" {
  */
 #define xmlSecXsltError(errorFunction, ctxt, errorObject) \
     {                                                 \
-        xmlErrorPtr error = xmlGetLastError();        \
+        const xmlError * error = xmlGetLastError();        \
         int code = (error != NULL) ? error->code : 0; \
         const char* message = (error != NULL) ? error->message : NULL; \
         xmlSecError(XMLSEC_ERRORS_HERE,               \
@@ -261,18 +262,34 @@ extern "C" {
 
 /**
  * xmlSecNotImplementedError:
- * @details:           the additional details.
+ * @msg:                the extra message.
  *
  * Macro. The XMLSec library macro for reporting "not implemented" errors.
  */
-#define xmlSecNotImplementedError(details) \
+#define xmlSecNotImplementedError(msg) \
         xmlSecError(XMLSEC_ERRORS_HERE,                     \
                     NULL,                                   \
                     NULL,                                   \
                     XMLSEC_ERRORS_R_NOT_IMPLEMENTED,        \
                     "details=%s",                           \
-                    xmlSecErrorsSafeString(details)         \
+                    xmlSecErrorsSafeString(msg)         \
         )
+
+/**
+ * xmlSecNotImplementedError2:
+ * @msg:                the extra message.
+ * @param:              the extra message param.
+ *
+ * Macro. The XMLSec library macro for reporting "not implemented" errors.
+ */
+#define xmlSecNotImplementedError2(msg, param) \
+        xmlSecError(XMLSEC_ERRORS_HERE,                     \
+                    NULL,                                   \
+                    NULL,                                   \
+                    XMLSEC_ERRORS_R_NOT_IMPLEMENTED,        \
+                    (msg), (param)                          \
+        )
+
 /**
  * xmlSecInvalidSizeError:
  * @name:               the name of the variable, parameter, etc.
@@ -949,7 +966,7 @@ extern "C" {
  *
  * Macro. The XMLSec library macro for reporting other XMLSec errors.
  */
-#define xmlSecOtherError2(code, errorObject, msg, param) \
+#define xmlSecOtherError2(code, errorObject, msg, param)    \
         xmlSecError(XMLSEC_ERRORS_HERE,                     \
                     (const char*)(errorObject),             \
                     NULL,                                   \
@@ -1014,6 +1031,10 @@ extern "C" {
                     (msg), (param1), (param2), (param3), (param4) \
         )
 
+
+XMLSEC_EXPORT int   xmlSecErrorsPrintCryptoLibraryLogOnExitIsEnabled    (void);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
