@@ -90,13 +90,13 @@ bool ClarisWksDbaseContent::getExtrema(MWAWVec2i &min, MWAWVec2i &max) const
   if (m_idColumnMap.empty())
     return false;
   bool first=true;
-  for (auto it : m_idColumnMap) {
+  for (auto const &it : m_idColumnMap) {
     int col=it.first;
     Column const &column=it.second;
     if (column.m_idRecordMap.empty())
       continue;
     max[0]=col;
-    for (auto rIt : column.m_idRecordMap) {
+    for (auto const &rIt : column.m_idRecordMap) {
       int row=rIt.first;
       if (first) {
         min[0]=col;
@@ -116,10 +116,10 @@ void ClarisWksDbaseContent::updateCellPositionsSet() const
 {
   if (!m_positionSet.empty() || m_idColumnMap.empty())
     return;
-  for (auto it : m_idColumnMap) {
+  for (auto const &it : m_idColumnMap) {
     int col=it.first;
     Column const &column=it.second;
-    for (auto rIt : column.m_idRecordMap) {
+    for (auto const &rIt : column.m_idRecordMap) {
       int row=rIt.first;
       m_positionSet.insert(MWAWVec2i(col,row));
     }
@@ -132,13 +132,10 @@ bool ClarisWksDbaseContent::getRecordList(std::vector<int> &list) const
   if (m_idColumnMap.empty())
     return false;
   std::set<int> set;
-  for (auto it : m_idColumnMap) {
+  for (auto const &it : m_idColumnMap) {
     Column const &column=it.second;
-    for (auto rIt : column.m_idRecordMap) {
-      int row=rIt.first;
-      if (set.find(row)==set.end())
-        set.insert(row);
-    }
+    for (auto const &rIt : column.m_idRecordMap)
+      set.insert(rIt.first);
   }
   if (set.empty())
     return false;
