@@ -87,7 +87,7 @@ bool MWAWCell::Format::getNumberingProperties(librevenge::RVNGPropertyList &prop
     propList.insert("librevenge:value-type", "boolean");
     break;
   case F_NUMBER:
-    if (m_digits>0)
+    if (m_digits>-1000)
       propList.insert("number:decimal-places", m_digits);
     if (m_thousandHasSeparator)
       propList.insert("number:grouping", true);
@@ -220,7 +220,7 @@ std::ostream &operator<<(std::ostream &o, MWAWCell::Format const &format)
   }
   o << ",";
 
-  if (format.m_digits != -1) o << "digits=" << format.m_digits << ",";
+  if (format.m_digits > -1000) o << "digits=" << format.m_digits << ",";
   if (format.m_integerDigits != -1) o << "digits[min]=" << format.m_integerDigits << ",";
   if (format.m_numeratorDigits != -1) o << "digits[num]=" << format.m_numeratorDigits << ",";
   if (format.m_denominatorDigits != -1) o << "digits[den]=" << format.m_denominatorDigits << ",";
@@ -327,6 +327,9 @@ void MWAWCell::addTo(librevenge::RVNGPropertyList &propList, std::shared_ptr<MWA
 #endif
     MWAW_DEBUG_MSG(("MWAWCell::addTo: called with unknown valign=%d\n", vAlignment()));
   }
+  int rot=int(m_rotation); // int seems better than double
+  if (rot)
+    propList.insert("style:rotation-angle", rot);
 }
 
 std::string MWAWCell::getColumnName(int col)
@@ -478,6 +481,8 @@ std::ostream &operator<<(std::ostream &o, MWAWCell const &cell)
     break;
 #endif
   }
+  if (cell.m_rotation<0 || cell.m_rotation>0)
+    o << "rot=" << cell.m_rotation << ",";
   if (cell.m_extraLine!=MWAWCell::E_None)
     o << cell.m_extraLineType << ",";
   return o;
@@ -603,7 +608,7 @@ bool MWAWCellContent::double2String(double val, MWAWCell::Format const &format,
     else s << "false";
     break;
   case MWAWCell::F_NUMBER:
-    if (format.m_digits>=0 && format.m_numberFormat!=MWAWCell::F_NUMBER_GENERIC)
+    if (format.m_digits>-1000 && format.m_numberFormat!=MWAWCell::F_NUMBER_GENERIC)
       s << std::setprecision(format.m_digits);
     switch (format.m_numberFormat) {
     case MWAWCell::F_NUMBER_CURRENCY:
@@ -705,7 +710,7 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent const &content)
   break;
   case MWAWCellContent::C_FORMULA:
     o << ",formula=";
-    for (auto &form : content.m_formula)
+    for (auto const &form : content.m_formula)
       o << form;
     if (content.isValueSet()) o << "[" << content.m_value << "]";
     break;
@@ -752,6 +757,13 @@ librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyLis
     pList.insert("librevenge:text",finalStr);
     break;
   }
+  case F_Unicode:
+    pList.insert("librevenge:type","librevenge-text");
+    if (m_content.c_str())
+      pList.insert("librevenge:text",librevenge::RVNGString(m_content.c_str()));
+    else
+      pList.insert("librevenge:text","");
+    break;
   case F_Double:
     pList.insert("librevenge:type","librevenge-number");
     pList.insert("librevenge:number",m_doubleValue, librevenge::RVNG_GENERIC);
@@ -762,29 +774,43 @@ librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyLis
     break;
   case F_Cell:
     pList.insert("librevenge:type","librevenge-cell");
-    pList.insert("librevenge:column",m_position[0][0]);
-    pList.insert("librevenge:row",m_position[0][1]);
-    pList.insert("librevenge:column-absolute",!m_positionRelative[0][0]);
-    pList.insert("librevenge:row-absolute",!m_positionRelative[0][1]);
-    if (!m_sheet.empty())
-      pList.insert("librevenge:sheet-name",m_sheet.c_str());
+    if (m_position[0][0]>=0) {
+      pList.insert("librevenge:column",m_position[0][0]);
+      pList.insert("librevenge:column-absolute",!m_positionRelative[0][0]);
+    }
+    if (m_position[0][1]>=0) {
+      pList.insert("librevenge:row",m_position[0][1]);
+      pList.insert("librevenge:row-absolute",!m_positionRelative[0][1]);
+    }
+    if (!m_sheet[0].empty())
+      pList.insert("librevenge:sheet-name",m_sheet[0]);
     if (!m_fileName.empty())
-      pList.insert("librevenge:file-name",m_fileName.c_str());
+      pList.insert("librevenge:file-name",m_fileName.cstr());
     break;
   case F_CellList:
     pList.insert("librevenge:type","librevenge-cells");
-    pList.insert("librevenge:start-column",m_position[0][0]);
-    pList.insert("librevenge:start-row",m_position[0][1]);
-    pList.insert("librevenge:start-column-absolute",!m_positionRelative[0][0]);
-    pList.insert("librevenge:start-row-absolute",!m_positionRelative[0][1]);
-    pList.insert("librevenge:end-column",m_position[1][0]);
-    pList.insert("librevenge:end-row",m_position[1][1]);
-    pList.insert("librevenge:end-column-absolute",!m_positionRelative[1][0]);
-    pList.insert("librevenge:end-row-absolute",!m_positionRelative[1][1]);
-    if (!m_sheet.empty())
-      pList.insert("librevenge:sheet-name",m_sheet.c_str());
+    if (m_position[0][0]>=0) {
+      pList.insert("librevenge:start-column",m_position[0][0]);
+      pList.insert("librevenge:start-column-absolute",!m_positionRelative[0][0]);
+    }
+    if (m_position[0][1]>=0) {
+      pList.insert("librevenge:start-row",m_position[0][1]);
+      pList.insert("librevenge:start-row-absolute",!m_positionRelative[0][1]);
+    }
+    if (m_position[1][0]>=0) {
+      pList.insert("librevenge:end-column",m_position[1][0]);
+      pList.insert("librevenge:end-column-absolute",!m_positionRelative[1][0]);
+    }
+    if (m_position[1][1]>=0) {
+      pList.insert("librevenge:end-row",m_position[1][1]);
+      pList.insert("librevenge:end-row-absolute",!m_positionRelative[1][1]);
+    }
+    if (!m_sheet[0].empty())
+      pList.insert("librevenge:sheet-name",m_sheet[0]);
+    if (!m_sheet[1].empty())
+      pList.insert("librevenge:end-sheet-name",m_sheet[1]);
     if (!m_fileName.empty())
-      pList.insert("librevenge:file-name",m_fileName.c_str());
+      pList.insert("librevenge:file-name",m_fileName.cstr());
     break;
 #if !defined(__clang__)
   default:
@@ -801,8 +827,8 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent::FormulaInstruction co
   else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Long)
     o << inst.m_longValue;
   else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Cell) {
-    if (!inst.m_sheet.empty()) o << inst.m_sheet;
-    if (!inst.m_fileName.empty()) o << "[" << inst.m_fileName << "]";
+    if (!inst.m_sheet[0].empty()) o << "\"" << inst.m_sheet[0].cstr() << "\"";
+    if (!inst.m_fileName.empty()) o << "[" << inst.m_fileName.cstr() << "]";
     if (!inst.m_positionRelative[0][0]) o << "$";
     if (inst.m_position[0][0]<0) o << "C" << inst.m_position[0][0];
     else {
@@ -814,9 +840,10 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent::FormulaInstruction co
     else o << inst.m_position[0][1];
   }
   else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_CellList) {
-    if (!inst.m_sheet.empty()) o << inst.m_sheet;
-    if (!inst.m_fileName.empty()) o << "[" << inst.m_fileName << "]";
+    if (!inst.m_fileName.empty()) o << "[" << inst.m_fileName.cstr() << "]";
     for (int l=0; l<2; ++l) {
+      if (!inst.m_sheet[l].empty() && (l==0 || inst.m_sheet[0]!=inst.m_sheet[1]))
+        o << "\"" << inst.m_sheet[l].cstr() << "\"";
       if (!inst.m_positionRelative[l][0]) o << "$";
       if (inst.m_position[l][0]<0) o << "C" << inst.m_position[l][0];
       else {
@@ -829,7 +856,8 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent::FormulaInstruction co
       if (l==0) o << ":";
     }
   }
-  else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Text)
+  else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Text ||
+           inst.m_type==MWAWCellContent::FormulaInstruction::F_Unicode)
     o << "\"" << inst.m_content << "\"";
   else
     o << inst.m_content;
