@@ -212,7 +212,7 @@ struct State
 	//! returns the color corresponding to an id
 	bool getColor(int id, WPSColor &color) const;
 	//! returns the pattern corresponding to a pattern id between 0 and 24
-	bool getPattern24(int id, WPSGraphicStyle::Pattern &pattern) const;
+	static bool getPattern24(int id, WPSGraphicStyle::Pattern &pattern);
 	//! returns the pattern corresponding to a pattern id between 0 and 32
 	bool getPattern32(int id, WPSGraphicStyle::Pattern &pattern);
 	//! the file version
@@ -322,8 +322,8 @@ bool State::getColor(int id, WPSColor &color) const
 			0x242424, 0x181818, 0/*checkme*/, 0/*checkme*/,
 			0/*checkme*/, 0/*checkme*/, 0/*checkme*/, 0/*checkme*/,
 		};
-		for (int i=0; i<int(WPS_N_ELEMENTS(quattroColorMap)); ++i)
-			THIS->m_colorsList.push_back(WPSColor(quattroColorMap[i]));
+		for (uint32_t i : quattroColorMap)
+			THIS->m_colorsList.push_back(WPSColor(i));
 	}
 	if (id < 0 || id >= int(m_colorsList.size()))
 	{
@@ -334,7 +334,7 @@ bool State::getColor(int id, WPSColor &color) const
 	return true;
 }
 
-bool State::getPattern24(int id, WPSGraphicStyle::Pattern &pat) const
+bool State::getPattern24(int id, WPSGraphicStyle::Pattern &pat)
 {
 	if (id<0 || id>24)
 	{
@@ -779,7 +779,7 @@ bool Quattro9Graph::readShapeRec(std::shared_ptr<WPSStream> const &stream, long
 				// we must update the style here
 				if (patId==0)
 					style.setSurfaceColor(surfColor[0]);
-				else if (patId>0)
+				else
 				{
 					WPSGraphicStyle::Pattern pattern;
 					if (m_state->getPattern32(patId, pattern))
@@ -1717,7 +1717,7 @@ bool Quattro9Graph::sendPageGraphics(int sheetId) const
 {
 	for (auto it=m_state->m_sheetIdToGraphMap.lower_bound(sheetId); it!=m_state->m_sheetIdToGraphMap.upper_bound(sheetId); ++it)
 	{
-		auto &graph=it->second;
+		auto const &graph=it->second;
 		if (!graph) continue;
 		if (graph->m_type==graph->OLE)
 			sendOLE(*graph, sheetId);
