@@ -19,10 +19,10 @@ namespace {
 
 namespace trf {
 
-using map_type = mdds::sorted_string_map<totals_row_function_t, mdds::string_view_map_entry>;
+using map_type = mdds::sorted_string_map<totals_row_function_t>;
 
 // Keys must be sorted.
-constexpr map_type::entry entries[] = {
+constexpr map_type::entry_type entries[] = {
     { "average",   totals_row_function_t::average },
     { "count",     totals_row_function_t::count },
     { "countNums", totals_row_function_t::count_numbers },
@@ -45,10 +45,10 @@ const map_type& get()
 
 namespace pc_group_by {
 
-using map_type = mdds::sorted_string_map<pivot_cache_group_by_t, mdds::string_view_map_entry>;
+using map_type = mdds::sorted_string_map<pivot_cache_group_by_t>;
 
 // Keys must be sorted.
-constexpr map_type::entry entries[] = {
+constexpr map_type::entry_type entries[] = {
     { "days",     pivot_cache_group_by_t::days },
     { "hours",    pivot_cache_group_by_t::hours },
     { "minutes",  pivot_cache_group_by_t::minutes },
@@ -69,10 +69,10 @@ const map_type& get()
 
 namespace error_value {
 
-using map_type = mdds::sorted_string_map<error_value_t, mdds::string_view_map_entry>;
+using map_type = mdds::sorted_string_map<error_value_t>;
 
 // Keys must be sorted.
-constexpr map_type::entry entries[] =
+constexpr map_type::entry_type entries[] =
 {
     { "#DIV/0!", error_value_t::div0  },
     { "#N/A!",   error_value_t::na    },
@@ -93,9 +93,9 @@ const map_type& get()
 
 namespace named_colors {
 
-using map_type = mdds::sorted_string_map<color_rgb_t, mdds::string_view_map_entry>;
+using map_type = mdds::sorted_string_map<color_rgb_t>;
 
-constexpr map_type::entry entries[] =
+constexpr map_type::entry_type entries[] =
 {
     { "aliceblue", { 0xF0, 0xF8, 0xFF } },
     { "antiquewhite", { 0xFA, 0xEB, 0xD7 } },
@@ -247,10 +247,10 @@ const map_type& get()
 
 namespace formula_error_policy {
 
-using map_type = mdds::sorted_string_map<formula_error_policy_t, mdds::string_view_map_entry>;
+using map_type = mdds::sorted_string_map<formula_error_policy_t>;
 
 // Keys must be sorted.
-constexpr map_type::entry entries[] = {
+constexpr map_type::entry_type entries[] = {
     { "fail", formula_error_policy_t::fail },
     { "skip", formula_error_policy_t::skip },
 };
@@ -411,6 +411,12 @@ std::ostream& operator<< (std::ostream& os, const src_address_t& v)
     return os;
 }
 
+std::ostream& operator<< (std::ostream& os, const src_range_t& v)
+{
+    os << v.first << "->" << v.last;
+    return os;
+}
+
 std::ostream& operator<< (std::ostream& os, const range_t& v)
 {
     os << v.first << "-" << v.last;
@@ -552,21 +558,21 @@ std::ostream& operator<< (std::ostream& os, border_style_t border)
         "unknown",
         "none",
         "solid",
-        "dash_dot",
-        "dash_dot_dot",
+        "dash-dot",
+        "dash-dot-dot",
         "dashed",
         "dotted",
-        "double_border",
+        "double-border",
         "hair",
         "medium",
-        "medium_dash_dot",
-        "medium_dash_dot_dot",
-        "medium_dashed",
-        "slant_dash_dot",
+        "medium-dash-dot",
+        "medium-dash-dot-dot",
+        "medium-dashed",
+        "slant-dash-dot",
         "thick",
         "thin",
-        "double_thin",
-        "fine_dashed",
+        "double-thin",
+        "fine-dashed",
     };
 
     return write_name_for_pos(os, names, std::size(names), border);
@@ -576,7 +582,7 @@ std::ostream& operator<< (std::ostream& os, formula_grammar_t grammar)
 {
     static constexpr std::string_view names[] = {
         "unknown",
-        "xls_xml",
+        "xls-xml",
         "xlsx",
         "ods",
         "gnumeric"
@@ -585,26 +591,23 @@ std::ostream& operator<< (std::ostream& os, formula_grammar_t grammar)
     return write_name_for_pos(os, names, std::size(names), grammar);
 }
 
-std::ostream& operator<< (std::ostream& os, underline_t uline)
+std::ostream& operator<< (std::ostream& os, underline_style_t uline)
 {
     static constexpr std::string_view names[] = {
         "none",
-        "single_line",
-        "single_accounting",
-        "double_line",
-        "double_accounting",
+        "solid",
         "dotted",
         "dash",
-        "long_dash",
-        "dot_dash",
-        "dot_dot_dash",
+        "long-dash",
+        "dot-dash",
+        "dot-dot-dash",
         "wave",
     };
 
     return write_name_for_pos(os, names, std::size(names), uline);
 }
 
-std::ostream& operator<< (std::ostream& os, underline_width_t ulwidth)
+std::ostream& operator<< (std::ostream& os, underline_thickness_t ulwidth)
 {
     static constexpr std::string_view names[] = {
         "none",
@@ -615,29 +618,30 @@ std::ostream& operator<< (std::ostream& os, underline_width_t ulwidth)
         "thick",
         "thin",
         "percent",
-        "positive_integer",
-        "positive_length",
+        "positive-integer",
+        "positive-length",
     };
 
     return write_name_for_pos(os, names, std::size(names), ulwidth);
 }
 
-std::ostream& operator<< (std::ostream& os, underline_mode_t ulmode)
+std::ostream& operator<< (std::ostream& os, underline_spacing_t ulmode)
 {
     static constexpr std::string_view names[] = {
         "continuous",
-        "skip_white_space",
+        "skip-white-space",
+        "continuous-over-field",
     };
 
     return write_name_for_pos(os, names, std::size(names), ulmode);
 }
 
-std::ostream& operator<< (std::ostream& os, underline_type_t ultype)
+std::ostream& operator<< (std::ostream& os, underline_count_t ultype)
 {
     static constexpr std::string_view names[] = {
         "none",
-        "single_type",
-        "double_type",
+        "single-count",
+        "double-count",
     };
 
     return write_name_for_pos(os, names, std::size(names), ultype);
@@ -672,6 +676,46 @@ std::ostream& operator<< (std::ostream& os, ver_alignment_t valign)
     return write_name_for_pos(os, names, std::size(names), valign);
 }
 
+std::ostream& operator<< (std::ostream& os, auto_filter_node_op_t op)
+{
+    static constexpr std::string_view names[] = {
+        "unspecified",
+        "op-and",
+        "op-or",
+    };
+
+    return write_name_for_pos(os, names, std::size(names), op);
+}
+
+std::ostream& operator<< (std::ostream& os, auto_filter_op_t op)
+{
+    static constexpr std::string_view names[] = {
+        "unspecified",
+        "empty",
+        "not-empty",
+        "equal",
+        "not-equal",
+        "contain",
+        "not-contain",
+        "begin-with",
+        "not-begin-with",
+        "end-with",
+        "not-end-with",
+        "greater",
+        "greater-equal",
+        "less",
+        "less-equal",
+        "top",
+        "bottom",
+        "top-percent",
+        "bottom-percent",
+        "top-percent-range",
+        "bottom-percent-range",
+    };
+
+    return write_name_for_pos(os, names, std::size(names), op);
+}
+
 std::ostream& operator<< (std::ostream& os, const color_rgb_t& color)
 {
     os << "(r=" << (int)color.red << ",g=" << (int)color.green << ",b=" << (int)color.blue << ")";
@@ -683,23 +727,23 @@ std::ostream& operator<< (std::ostream& os, const fill_pattern_t& fill)
     static constexpr std::string_view names[] = {
         "none",
         "solid",
-        "dark_down",
-        "dark_gray",
-        "dark_grid",
-        "dark_horizontal",
-        "dark_trellis",
-        "dark_up",
-        "dark_vertical",
-        "gray_0625",
-        "gray_125",
-        "light_down",
-        "light_gray",
-        "light_grid",
-        "light_horizontal",
-        "light_trellis",
-        "light_up",
-        "light_vertical",
-        "medium_gray",
+        "dark-down",
+        "dark-gray",
+        "dark-grid",
+        "dark-horizontal",
+        "dark-trellis",
+        "dark-up",
+        "dark-vertical",
+        "gray-0625",
+        "gray-125",
+        "light-down",
+        "light-gray",
+        "light-grid",
+        "light-horizontal",
+        "light-trellis",
+        "light-up",
+        "light-vertical",
+        "medium-gray",
     };
 
     return write_name_for_pos(os, names, std::size(names), fill);
@@ -711,10 +755,10 @@ std::ostream& operator<< (std::ostream& os, const strikethrough_style_t& ss)
         "none",
         "solid",
         "dash",
-        "dot_dash",
-        "dot_dot_dash",
+        "dot-dash",
+        "dot-dot-dash",
         "dotted",
-        "long_dash",
+        "long-dash",
         "wave",
     };
 
@@ -726,8 +770,8 @@ std::ostream& operator<< (std::ostream& os, const strikethrough_type_t& st)
     static constexpr std::string_view names[] = {
         "unknown",
         "none",
-        "single_type",
-        "double_type",
+        "single-type",
+        "double-type",
     };
 
     return write_name_for_pos(os, names, std::size(names), st);
@@ -737,7 +781,7 @@ std::ostream& operator<< (std::ostream& os, const strikethrough_width_t& sw)
 {
     static constexpr std::string_view names[] = {
         "unknown",
-        "width_auto",
+        "width-auto",
         "thin",
         "medium",
         "thick",
@@ -758,6 +802,123 @@ std::ostream& operator<< (std::ostream& os, const strikethrough_text_t& st)
     return write_name_for_pos(os, names, std::size(names), st);
 }
 
+std::ostream& operator<<(std::ostream& os, pivot_cache_group_by_t v)
+{
+    constexpr std::string_view values[] = {
+        "unknown",
+        "days",
+        "hours",
+        "minutes",
+        "months",
+        "quarters",
+        "range",
+        "seconds",
+        "years",
+    };
+
+    if (auto pos = std::size_t(v); pos < std::size(values))
+        os << values[pos];
+    else
+        os << "???";
+
+    return os;
+}
+
+std::ostream& operator<<(std::ostream& os, pivot_field_item_t v)
+{
+    constexpr std::string_view values[] = {
+        "unknown",
+        "average",
+        "blank-line",
+        "count",
+        "count-numbers",
+        "data",
+        "subtotal-default",
+        "grand-total",
+        "max",
+        "min",
+        "product",
+        "stddev",
+        "stddevp",
+        "sum",
+        "var",
+        "varp",
+    };
+
+    if (auto pos = std::size_t(v); pos < std::size(values))
+        os << values[pos];
+    else
+        os << "???";
+
+    return os;
+}
+
+std::ostream& operator<<(std::ostream& os, pivot_axis_t v)
+{
+    constexpr std::string_view values[] = {
+        "unknown",
+        "column",
+        "page",
+        "row",
+        "values",
+    };
+
+    if (auto pos = std::size_t(v); pos < std::size(values))
+        os << values[pos];
+    else
+        os << "???";
+
+    return os;
+}
+
+std::ostream& operator<<(std::ostream& os, pivot_data_show_data_as_t v)
+{
+    constexpr std::string_view values[] = {
+        "unknown",
+        "difference",
+        "index",
+        "normal",
+        "percent",
+        "percent-diff",
+        "percent-of-col",
+        "percent-of-row",
+        "percent-of-total",
+        "run-total",
+    };
+
+    if (auto pos = std::size_t(v); pos < std::size(values))
+        os << values[pos];
+    else
+        os << "???";
+
+    return os;
+}
+
+std::ostream& operator<<(std::ostream& os, pivot_data_subtotal_t v)
+{
+    constexpr std::string_view values[] = {
+        "unknown",
+        "average",
+        "count",
+        "count-numbers",
+        "max",
+        "min",
+        "product",
+        "stddev",
+        "stddevp",
+        "sum",
+        "var",
+        "varp",
+    };
+
+    if (auto pos = std::size_t(v); pos < std::size(values))
+        os << values[pos];
+    else
+        os << "???";
+
+    return os;
+}
+
 }}
 
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
