@@ -12,6 +12,7 @@
 #include <cstring>
 #include <vector>
 #include <string>
+#include <iterator>
 
 using namespace orcus;
 using namespace std;
@@ -20,17 +21,13 @@ void test_base64_text_input(const char* p)
 {
     cout << "input: '" << p << "'" << endl;
     size_t n = strlen(p);
-    vector<char> input(p, p+n);
-    string encoded;
-    encode_to_base64(input, encoded);
+    std::vector<uint8_t> input(p, p+n);
+    std::string encoded = encode_to_base64(input);
     cout << "encoded: '" << encoded << "'" << endl;
 
-    vector<char> decoded;
-    decode_from_base64(&encoded[0], encoded.size(), decoded);
+    std::vector<uint8_t> decoded = decode_from_base64(encoded);
     cout << "decoded: '";
-    vector<char>::const_iterator it = decoded.begin(), it_end = decoded.end();
-    for (; it != it_end; ++it)
-        cout << *it;
+    std::copy(decoded.begin(), decoded.end(), std::ostream_iterator<char>(cout, ""));
     cout << "'" << endl;
 
     assert(input == decoded);
