@@ -1,4 +1,4 @@
-// Copyright 2021 PDFium Authors. All rights reserved.
+// Copyright 2021 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,13 +8,13 @@
 #include <time.h>
 
 #include "build/build_config.h"
-#include "third_party/base/check_op.h"
+#include "core/fxcrt/check_op.h"
 
 namespace {
 
 constexpr char kTZ[] = "TZ";
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 #define SETENV(name, value) _putenv_s(name, value)
 #define TZSET _tzset
 #define UNSETENV(name) _putenv_s(name, "")
