@@ -18,7 +18,7 @@
 #include "include/core/SkSurface.h"
 #include "include/core/SkTypes.h"
 #include "include/private/base/SkDebug.h"
-#include "include/private/base/SkFloatBits.h"
+#include "src/base/SkFloatBits.h"
 #include "src/core/SkCubicClipper.h"
 #include "tests/Test.h"
 
