@@ -16,6 +16,7 @@
 #include "include/private/base/SkTArray.h"
 #include "modules/sksg/include/SkSGGroup.h"
 #include "modules/sksg/include/SkSGScene.h"
+#include "tools/timer/TimeUtils.h"
 #include "tools/viewer/Slide.h"
 
 #include <memory>
@@ -67,7 +68,7 @@ private:
 
     SkSize                             fWinSize  = SkSize::MakeEmpty();
     SkSize                             fCellSize = SkSize::MakeEmpty();
-    SkMSec                             fTimeBase = 0;
+    TimeUtils::MSec                  fTimeBase = 0;
 
     const Rec*                         fTrackingCell = nullptr;
     SkPoint                            fTrackingPos  = SkPoint::Make(0, 0);
